/*
 *  playlist.h
 *  mod_musicindex
 *
 *  $Id: playlist.h,v 1.26 2004/09/11 16:56:06 varenet Exp $
 *
 *  Created by Thibaut VARENE on Thu Mar 20 2003.
 *  Copyright (c) 2003-2004 Regis BOUDIN
 *  Copyright (c) 2003-2004 Thibaut VARENE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */

#ifndef PLAYLIST_H
#define PLAYLIST_H

#include "mod_musicindex.h"
#include <stdio.h>	/* fops */

/**
 * Function pointer type, as we need it.
 *
 * Here comes the declaration of the funny part... We will (again) use
 * function pointers which will allocate a new entry  from the apache pool,
 * fill it, and close the "in" file if they recognise the file.
 */
typedef mu_ent* (*make_entry_ptr)(apr_pool_t*, mu_ent *, FILE *,
				mu_config *, mu_ent_names *, request_rec *);

/**
 * Enumerate handlers to access relevant data in struct filetype.
 * <b>BEWARE</b>: you must keep that in a coherent order wrt
 * struct filetype[] !
 */
enum { FT_MP3, FT_OGG, FT_FLAC, FT_MP4 };

/**
 * Array of common strings widely used.
 * Mind the above enum comment please.
 *
 * http://filext.com/detaillist.php?extdetail=M3U
 * http://filext.com/detaillist.php?extdetail=MP3
 * http://filext.com/detaillist.php?extdetail=OGG
 * http://filext.com/detaillist.php?extdetail=FLAC
 * http://filext.com/detaillist.php?extdetail=MP4
 */
static const struct {
	const char *nametype;	/**< Display name of the filetype */
	const char *mimetype;	/**< Mime type associated to the filetype */
} filetype[] = {
	{ "MP3", "audio/x-mp3" },
	{ "Vorbis", "audio/x-ogg" },
	{ "FLAC", "audio/flac" },
	{ "MP4/AAC", "audio/mp4" }
};

mu_ent *build_custom_list(request_rec *r, mu_config *conf);

mu_ent *new_ent(apr_pool_t *pool, mu_ent *head);

mu_ent *make_music_entry(apr_pool_t *pool, request_rec *r,
	mu_ent *head, mu_config *conf, mu_ent_names *names);

#endif	/* PLAYLIST_H */
